##
## Copyright (c) 2006 by Rafael D'Halleweyn
##
## Licensed under Version 2 of the GPL
##

package ClusTerm::Diff;

use strict;
use warnings;
use Exporter 'import';

our @EXPORT = qw(lcss split_rc);
our @EXPORT_OK = qw(lcss split_rc);

# longest common sub-sequence
sub lcss {
	my $get_key;
	if (ref $_[-1] eq 'CODE') {
		$get_key = pop @_;
	}

	my (@a) = @_;
	my @pos = (0) x @a;

	# for each text fragment
	# index{key} = {
	# 	idx => $idx,
	# 	pos_a => [ [ pos-in-a[0], ... ], [ pos-in-a[1] ] ]
	# 	pos_h => [ { pos-in-a[0] => 1, .. }, [ pos-in-a[1] => 1 ] ]
	# }
	my %index;
	# new arrays, mirror original mirror @a, but contains the index for
	# each item
	my @b;
	for my $i (0..$#pos) {
		for(my $j = 0; $j < @{$a[$i]}; $j++) {
			my $key = $a[$i][$j];
			if ($get_key) {
				local $_ = $key;
				$key = &$get_key;
			}
			my $idx;
			if (exists $index{$key}) {
				$idx = $index{$key};
			} else {
				$idx = $index{$key} = {
					idx => scalar keys %index,
					pos_a => [ ],
					pos_h => [ ],
				};
			}
			$b[$i][$j] = $idx;
			push @{$idx->{pos_a}[$i]}, $j;
			$idx->{pos_h}[$i]{$j} = $#{$idx->{pos_a}[$i]};
		}
	}

	# identify those that match in each sequence
	my $matches = 0;
IDX:	for my $idx (values %index) {
		for my $i (0..$#pos) {
			unless (defined $idx->{pos_h}[$i]
				&& keys %{$idx->{pos_h}[$i]}) {
					$idx->{match} = 0;
					next IDX;
			}
		}
		$idx->{match} = 1;
		$matches++;
	}

	return wantarray ? ( ) : 0 unless $matches;
	my $css = _lcss(\@b, @pos);
	return wantarray ? @$css : scalar @$css;
}

sub _lcss {
	my ($b, @pos) = @_;

	my @css;
POS:	while(1) {
		for my $i (0..$#pos) {
			last POS unless $pos[$i] < @{$b->[$i]};
		}

		my $posstr = join(":", @pos);
		my $idx = $b->[0][$pos[0]];
		if (exists $idx->{next}{$posstr}) {
			# we already know the best next position
			push @css, $idx->{next}{$posstr};
			next POS;
		}

		if ($idx->{match}) {
			for my $i (1..$#pos) {
				goto SEARCH
					unless exists
					$idx->{pos_h}[$i]{$pos[$i]};
			}

			push @css, [ @pos ];
			$idx->{next}{$posstr} = [ @pos ];
			next POS;
		}

		# find all the next possible matches
SEARCH:		my @match;
IDX:		for(my $j = $pos[0]; $j < @{$b->[0]}; $j++) {
			my $idx = $b->[0][$j];
			next unless $idx->{match};

			my @newpos = ( $j );
POSI:			for my $i (1..$#pos) {
				for (@{$idx->{pos_a}[$i]}) {
					next if $_ < $pos[$i];
					push @newpos, $_;
					next POSI;
				}
				next IDX;
			}

			# any position where all the coordinates are bigger
			# than or equal to the coordinates of a previous
			# match should not be considered
			# [this is a major work safer!!]
MATCH:			for my $m (@match) {
				for my $i (1..$#pos) {
					next MATCH if $newpos[$i] < $m->[$i];	
				}
				next IDX;
			}

			push @match, \@newpos;
		}

		# no further matches found
		last POS unless @match;

		# only one match found
		if (@match == 1) {
			push @css, $match[0];
			$idx->{next}{$posstr} = $match[0];
			next POS;
		}

		# find the best match
		my $best;
		my $best_css;
		for my $k (0..$#match) {
			my @newpos = map { $_ + 1 } @{$match[$k]};
			my $other_css = _lcss($b, @newpos);
			unless ($best_css && @$best_css >= @$other_css) {
				$best = $k;
				$best_css = $other_css;
			}
		}

		push @css, $match[$best], @$best_css;
		$idx->{next}{$posstr} = $match[$best];
		last POS;
	} continue {
		for my $i (0..$#pos) {
			$pos[$i] = $css[-1][$i] + 1;
		}
	}

	return \@css;
}

# split based on re, but remember row, column and offset in the original
# string
sub split_rc {
	my ($re, $t, $cols) = @_;
	$cols ||= ~0;

	my ($r, $c, $off) = (0, 0, 0);
	my @a = map {
		# if behind last column go to next row
		# if just behind last column and not newline also go to next row
		if ($c > $cols || (/^[^\n]/ && $c >= $cols)) {
			$c -= $cols;
			$r++;
		}

		my $a = { t => $_, r => $r, c => $c, off => $off };
		$off += length($_);
		if (/\n/) {
			while(s/^[^\n]*\n//) { $r++ }
			$c = length($_);
		} else {
			$c += length($_);
		}
		$a;
	} split /($re)/, $t;

	return \@a;
}



1;
