##
## Copyright (c) 2005-2007 by Rafael D'Halleweyn
##
## Licensed under Version 2 of the GPL
##

package ClusTerm::Menus;

use strict;
use warnings;
use Glib qw(TRUE FALSE);
use ClusTerm::Term qw(:roles);
use ClusTerm::App;

# reusable menu entries
our $self;
our %menu;
%menu = (
	separator => {
		type => 'separator'
	},
	new_host => {
		type => 'item', label => '_New Host',
		pixmap_type => 'stock', pixmap_info => 'gtk-new',
		accelerator => '<Shift><Ctrl>N',
		callback => \&new_host,
	},
	close_host => {
		type => 'item', label => '_Close Current Host',
		pixmap_type => 'stock', pixmap_info => 'gtk-close',
		accelerator => '<Shift><Ctrl>W',
		callback => \&ClusTerm::App::close_host,
		sensitive_when_terms => 1,
	},
	save_session_launcher => {
		type => 'item', label => '_Save Session As Launcher...',
		pixmap_type => 'stock', pixmap_info => 'gtk-save',
		callback => \&ClusTerm::App::save_session_launcher,
	},
	quit => {
		type => 'item', label => '_Quit',
		pixmap_type => 'stock', pixmap_info => 'gtk-quit',
		accelerator => '<Shift><Ctrl>Q',
		callback => sub { Gtk2->main_quit; },
	},
	copy =>	{
		type => 'item', label => '_Copy',
		pixmap_type => 'stock', pixmap_info => 'gtk-copy',
		accelerator => '<Shift><Ctrl>C',
		callback => \&ClusTerm::App::copy,
		sensitive_when_terms => 1,
	},
	paste => {
		type => 'item', label => '_Paste',
		pixmap_type => 'stock', pixmap_info => 'gtk-paste',
		accelerator => '<Shift><Ctrl>V',
		callback => \&ClusTerm::App::paste,
		sensitive_when_terms => 1,
	},
	become_director => {
		type => 'item', label => 'Become _Director',
		pixmap_type => 'icon', pixmap_info => 'gtk-media-record',
		accelerator => '<Shift><Ctrl>D',
		callback => sub { $_[0]->set_role(ROLE_DIRECTOR); },
		sensitive_when_terms => 1,
	},
	become_actor => {	
		type => 'item', label => 'Become Ac_tor',
		pixmap_type => 'icon', pixmap_info => 'gtk-media-play-ltr',
		accelerator => '<Shift><Ctrl>P',
		callback => sub { $_[0]->set_role(ROLE_ACTOR); },
		sensitive_when_terms => 1,
	},
	become_free => {
		type => 'item', label => 'Become _Free',
		pixmap_type => 'icon', pixmap_info => 'gtk-media-stop',
		accelerator => '<Shift><Ctrl>U',
		callback => sub { $_[0]->set_role(ROLE_FREE); },
		sensitive_when_terms => 1,
	},
	become_director_all => {
		type => 'item', label => 'Become Director of _All',
		accelerator => '<Shift><Ctrl>A',
		callback => sub { $_[0]->set_role(ROLE_DIRECTOR, TRUE); },
		sensitive_when_terms => 2,
	},
	insert_hostname => {
		type => 'item', label => 'Insert Hostname',
		accelerator => '<Shift><Ctrl>H',
		callback => \&ClusTerm::App::insert_hostname,
		sensitive_when_terms => 1,
	},
	preferences => {
		type => 'item', label => 'Prefere_nces',
		pixmap_type => 'stock', pixmap_info => 'gtk-preferences',
		callback => \&ClusTerm::App::preferences,
	},

	# The menu_bar on the context_menu both contain the same hash reference
	# (this one). We will store the menu widgets that implement it
	# in the widget key of the hash.
	# This stuff should be moved to a utility function
	show_menubar => {
		type => 'toggleitem', label => 'Show _Menubar',
		callback => sub {
			my ($self, $menu) = @_;
			my $show = $menu->get_active;
			unless (@{$self->{term}} || $show) {
				# don't hide menubar if there are no terminals
				# there would be nothing in the window
				$menu->set_active(1);
				return;
			}

			map { $_->set_active($show) }
				@{$menu{show_menubar}{widgets}};
			$self->on_show_menu($show);
		},
		sensitive_when_terms => 1,
	},
	view_as => {
		type => 'radioitems', moreinfo => [
			{ type => 'item', label => 'As Single _Row',
			  callback => sub {
			  	$_[0]->change_type('row', $_[1])
			  },
			  value => 'row',
			},
			{ type => 'item', label => 'As Single _Column',
			  callback => sub {
			  	$_[0]->change_type('column', $_[1])
			  },
			  value => 'column',
			},
			{ type => 'item', label => 'As _Table',
			  callback => sub {
			  	$_[0]->change_type('table', $_[1])
			  },
			  value => 'table',
			},
			{ type => 'item', label => 'As _Notebook',
			  callback => sub {
			  	$_[0]->change_type('notebook', $_[1])
			  },
			  value => 'notebook',
			},
		],
	},
	highlight_diffs => {
		type => 'item', label => 'Highlight Di_fferences',
		accelerator => '<Shift><Ctrl>F',
		callback => \&ClusTerm::App::highlight_diffs,
		sensitive_when_terms => 2,
	},
	remove_highlights => {
		type => 'item', label => 'Remove Highlight',
		accelerator => '<Shift><Ctrl>R',
		callback => \&ClusTerm::App::remove_highlights,
		sensitive_when_terms => 2,
	},
	previous_host => {
		type => 'item', label => '_Previous Host',
		accelerator => '<Shift><Ctrl>Left',
		pixmap_type => 'stock', pixmap_info => 'gtk-go-back',
		callback => sub { $_[0]->set_focus(-1) },
		sensitive_when_terms => 2,
	},
	next_host => {
		type => 'item', label => '_Next Host',
		accelerator => '<Shift><Ctrl>Right',
		pixmap_type => 'stock', pixmap_info => 'gtk-go-forward',
		callback => sub { $_[0]->set_focus(+1) },
		sensitive_when_terms => 2,
	},
	to_director => {
		type => 'item', label => 'To _Director',
		accelerator => '<Shift><Ctrl><Alt>D',
		callback => sub { $_[0]->set_focus($_[0]->{director}) },
		sensitive_when_terms => 2,
	},
	about => {
		type => 'item', label => '_About...',
		pixmap_type => 'stock', pixmap_info => 'gtk-about',
		callback => \&ClusTerm::App::about,
	},
	zoom_in => {
		type => 'item', label => 'Zoom _In',
		pixmap_type => 'stock', pixmap_info => 'gtk-zoom-in',
		accelerator => '<Ctrl>plus',
		callback => \&ClusTerm::App::zoom_in,
		sensitive_when_terms => 1,
	},
	zoom_out => {
		type => 'item', label => 'Zoom _Out',
		pixmap_type => 'stock', pixmap_info => 'gtk-zoom-out',
		accelerator => '<Ctrl>minus',
		callback => \&ClusTerm::App::zoom_out,
		sensitive_when_terms => 1,
	},
	zoom_normal => {
		type => 'item', label => 'N_ormal Size',
		pixmap_type => 'stock', pixmap_info => 'gtk-zoom-100',
		accelerator => '<Ctrl>equal',
		callback => \&ClusTerm::App::zoom_normal,
		sensitive_when_terms => 1,
	},
);

# predefined context-menus
our @context_menu = (
	$menu{new_host},
	$menu{separator},
	$menu{copy},
	$menu{paste},
	$menu{insert_hostname},
	$menu{separator},
	$menu{become_director},
	$menu{become_actor},
	$menu{become_free},
	$menu{become_director_all},
	$menu{separator},
	$menu{to_director},
	$menu{separator},
	$menu{show_menubar},
	$menu{separator},
	$menu{close_host},
);

# shortcuts for things that are not in any menu
our @shortcuts = (
);

# predefined menus
our @menus = (
	{ type => 'subtree-stock', label => '_File',
		subtree => [
			$menu{new_host},
			$menu{save_session_launcher},
			$menu{separator},
			$menu{close_host},
			$menu{quit},
		],
	},
	{ type => 'subtree-stock', label => '_Edit',
		subtree => [
			$menu{copy},
			$menu{paste},
			$menu{separator},
			$menu{become_director},
			$menu{become_actor},
			$menu{become_free},
			$menu{become_director_all},
			$menu{separator},
			$menu{insert_hostname},
			$menu{separator},
			$menu{preferences},
		],
	},
	{ type => 'subtree-stock', label => '_View',
		subtree => [
			$menu{show_menubar},
			$menu{separator},
			$menu{view_as},
			#$menu{separator},
			#{ type => 'item', label => 'Make Table _Wider', },
			#{ type => 'item', label => 'Make Table _Taller', },
			$menu{separator},
			$menu{zoom_in},
			$menu{zoom_out},
			$menu{zoom_normal},
			$menu{separator},
			$menu{highlight_diffs},
			$menu{remove_highlights},
		],
	},
	{ type => 'subtree-stock', label => '_Go',
		subtree => [
			$menu{previous_host},
			$menu{next_host},
			$menu{separator},
			$menu{to_director},
		],
	},
	{ type => 'subtree-stock', label => '_Help',
		subtree => [
			$menu{about},
		],
	},
);

my $new_host_dialog;
my $new_host_entry;

sub new_host {
	my $self = shift;

	if ($new_host_dialog) {
		$new_host_dialog->present;
		$new_host_entry->grab_focus;
		return;
	}

	# flags: modal, destroy-with-parent
	$new_host_dialog = Gtk2::Dialog->new("New Host", $main::app,
		[ 'destroy-with-parent', 'no-separator' ],
		'gtk-cancel' => 'cancel',
		'gtk-add' => 'ok');
	$new_host_dialog->set_default_response('ok');
	my $hbox = Gtk2::HBox->new;
	$hbox->set_spacing(6);
	$hbox->set_border_width(6);
	$new_host_dialog->vbox->add($hbox);

	my $tips = Gtk2::Tooltips->new();

	$hbox->pack_start(Gtk2::Label->new("Host:"), FALSE, FALSE, 0);
	$new_host_entry = Gtk2::Entry->new();
	$new_host_entry->set_activates_default(TRUE);
	$new_host_entry->grab_focus;
	$tips->set_tip($new_host_entry,
		"Enter the hostname to connect to.\n"
		."If the string starts with '/' or contains a\n"
		."space it will be executed as is.\n"
		."Otherwise the string will be used with the\n"
		."command configured in the preferences." );
	$hbox->pack_start($new_host_entry, TRUE, TRUE, 0);

	$new_host_dialog->signal_connect_swapped(
		response => \&new_host2, $self);
	$new_host_dialog->show_all;
}

sub new_host2 {
	my ($self, $i, $dialog) = @_;

	if ($i eq 'ok') {
		my $host = $new_host_entry->get_text;
		if ($host) {
			my @size = (80, 24);;
			@size = (
				$self->{term}[0]->get_column_count,
				$self->{term}[0]->get_row_count
			) if $self->{term}[0];
			$self->add_host($host);
			$self->set_my_geometry();
			$self->resize_from_cols_rows(@size, TRUE);
			$self->show_all;
		}
	}

	$new_host_dialog->destroy unless $i eq 'delete-event';
	$new_host_dialog = undef;
}

sub create_menu_bar {
	my $menu = Gtk2::MenuBar->new;
	my $accel = Gtk2::AccelGroup->new;
	$menu->fill_menu(\@menus, $accel, 0, 1);
	&ClusTerm::Utils::uiinfo_store_widgets(\@menus, $menu);

	return ($menu, $accel);
}

sub create_context_menu {
	my $menu = Gtk2::Menu->new;
	my $accel = Gtk2::AccelGroup->new;
	$menu->fill_menu(\@context_menu, $accel, 0, 1);
	&ClusTerm::Utils::uiinfo_store_widgets(\@context_menu, $menu);

	return ($menu, $accel);
}

sub create_shortcut_accel {
	my $accel = Gtk2::AccelGroup->new;
	for (@shortcuts) {
		next unless $_->{callback};
		$accel->connect($_->{accelerator_key}, $_->{ac_mods},
			[ ], $_->{callback});
	}
	return $accel;
}

1;
